{ 

  TPlacement component v1.01.  12 Feb 1999.
  Alexander Kopilovitch   e-mail: aek@vib.usr.pu.ru  or  aek@pcaek.spb.su
  
  Requires TKinComponentList object and property editor
  ( see http://www.tarkvara.com/components/kincomponentlist/kincompl.zip )

  TPlacement component is designed for remembering and restoring a placement
  of several controls (and their sizes).
  The list of controls is accumulated in TKinComponentList object.
  At design-time TPlacement component uses TKinComponentList Property Editor
  for gathering controls.
  At run-time the Apply method may be used for restoring a placement remembered
  by the component.  At design-time double-clicking the TPlacement component
  perfoms the same (actually the Apply method is called).

}

unit Placectl;

interface

uses
{$ifdef Win32}
   Windows,
{$else}
   WinTypes, WinProcs,
{$endif}
  SysUtils, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf, CompList;

type
  TPlacement = class(TComponent)
  private
    FMembers: TKinComponentList;
    FPlacements: TStringList;
    procedure SetMembers(M: TKinComponentList);
    procedure SetPlacements(P: TStringList);
  protected
    procedure Notification(Component: TComponent; Operation: TOperation); override;
    procedure Loaded; override;
    procedure RunTime_Setup(Sender: TObject);
    procedure DesignTime_Setup(Sender: TObject; ModalResult: TModalResult);
    procedure PickUp;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Apply;
  published
    property Members: TKinComponentList read FMembers write SetMembers;
    property Placements: TStringList read FPlacements write SetPlacements;
  end;

type TPlacementEditor = class(TComponentEditor)
  procedure Edit; override;
  end;  

procedure Register;

implementation

const Comma = ',';

procedure Register;
begin
  RegisterComponents('Samples', [TPlacement]);
  RegisterComponentEditor(TPlacement, TPlacementEditor);
end;

constructor TPlacement.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FMembers:=TKinComponentList.Create(Self, TControl, emReadWrite);
FMembers.OnChange:=RunTime_Setup;
FMembers.OnDesign:=DesignTime_Setup;
FPlacements:=TStringList.Create;
end;

destructor TPlacement.Destroy;
begin
FPlacements.Free;
FMembers.Free;
inherited Destroy;
end;

procedure TPlacement.SetMembers(M: TKinComponentList);
begin
{empty procedure}
end;

procedure TPlacement.SetPlacements(P: TStringList);
begin
if csDesigning in ComponentState then
  FPlacements.Assign(P);
end;

procedure TPlacement.Notification(Component: TComponent; Operation: TOperation); 
begin
inherited Notification(Component,Operation);
if (Component <> Self) and (Operation = opRemove) then
  FMembers.DeleteComponent(Component);
end;

procedure TPlacement.Loaded; 
begin
inherited Loaded;
FMembers.Load;
end;

procedure TPlacement.RunTime_Setup(Sender: TObject);
begin
if not (csDesigning in ComponentState) then
  PickUp;
end;

procedure TPlacement.DesignTime_Setup(Sender: TObject; ModalResult: TModalResult);
begin
if (csDesigning in ComponentState) and (ModalResult = mrOK) then
  PickUp;
end;

procedure TPlacement.PickUp;
var I: Integer;
    S: string;
    C: TComponent;
    D: Longint;
begin
FPlacements.Clear;
with FMembers do if Count > 0 then for I:=0 to Count-1 do begin
  C:=Items[I];
  if C = nil then
    S:=''
  else with C as TControl do
      S:=IntToStr(Top) + Comma + IntToStr(Left)
         + Comma + IntToStr(Height) + Comma + IntToStr(Width);
  FPlacements.Add(S);
  end;
end;

procedure TPlacement.Apply;
var I: Integer;
    XTop, XLeft, XHeight, XWidth: Integer;
    S: string;
    C: TComponent;
    D, D2: Longint;
  function ExtractNumber(var S: string) : Integer;
  var K: Integer;
      T: string;
  begin
  Result:=0;
  K:=Pos(Comma, S);
  if K > 0 then begin
    T:=Copy(S, 1, K-1);
    Delete(S, 1, K);
    end
  else begin
    T:=S;
    S:='';
    end;
  try
    Result:=StrToInt(T);
    except on EConvertError do
      ShowMessage('TPlacement.Apply: string "' + T + '" doesn''t represent an integer number'
                  + ', zero is substituted');
    end;
  end;
begin
with FMembers do if Count > 0 then for I:=0 to Count-1 do begin
  S:=FPlacements[I]; 
  if S = '' then  { not saved, was NIL during PickUp }
    Continue;
  XTop:=ExtractNumber(S);
  XLeft:=ExtractNumber(S);
  XHeight:=ExtractNumber(S);
  XWidth:=ExtractNumber(S);
  C:=Items[I];
  if C = nil then
    Continue;
  if C is TControl then with C as TControl do begin
    if (Top <> XTop) or (Left <> XLeft) or (Height <> XHeight) or (Width <> XWidth) then
      SetBounds(XLeft, XTop, XWidth, XHeight);
    end;  
  end;
end;

procedure TPlacementEditor.Edit;
begin
(Component as TPlacement).Apply;
end;

end.
 